### A Pluto.jl notebook ###
# v0.19.26

using Markdown
using InteractiveUtils

# ╔═╡ 8b2c2972-ca22-41a6-8bd5-bce1431ead8b
using HypertextLiteral

# ╔═╡ 88f021e1-b626-42a8-b88a-e90cc807fa9f
using Random: randstring

# ╔═╡ 0283cdf9-5dbf-4423-a298-357641f0d480
import AbstractPlutoDingetjes.Bonds

# ╔═╡ 6a14468f-2d51-4630-a4f3-eeae8a7ca912
begin
	struct Radio
	    options::Vector{Pair{<:AbstractString,<:Any}}
	    default::Union{Nothing, AbstractString}
		orientation::Symbol
	end
	Radio(options::AbstractVector{<:AbstractString}; default=nothing, orientation=:vertical) = Radio([o => o for o in options], default, orientation)
	Radio(options::AbstractVector{<:Pair{<:AbstractString,<:Any}}; default=nothing, orientation=:vertical) = Radio(options, default, orientation)
end

# ╔═╡ dabfd200-ad92-42f0-ab12-cc75e2253b90
function Base.show(io::IO, m::MIME"text/html", radio::Radio)
    groupname = randstring('a':'z')
		
	h = @htl(
		"""<form style="$(radio.orientation == :horizontal ? "display: flex" : nothing)">$(
		map(radio.options) do o
			@htl(
				"""<div><input $((
					type="radio",
					id=(groupname * o.first),
					name=groupname,
					value=o.first,
					checked=(radio.default === o.first),
				))><label for=$(groupname * o.first)>$(
					o.second
				)</label></div>"""
			)
        end	
		)<script>
		const form = currentScript.parentElement
		const groupname = $(groupname)
		
        const selected_radio = form.querySelector('input[checked]')

		let val = selected_radio?.value

		Object.defineProperty(form, "value", {
			get: () => val,
			set: (newval) => {
				val = newval
				const i = document.getElementById(groupname + newval)
				if(i != null){
					i.checked = true
				}
			},
		})

        form.oninput = (e) => {
            val = e.target.value
            // and bubble upwards
        }
		</script></form>""")
	show(io, m, h)
end

# ╔═╡ 4708c993-9cc3-43df-9984-57b97171e1a3
Base.get(radio::Radio) = radio.default

# ╔═╡ 081e6a06-0a6d-49b5-ae1a-05c20f0bd924
Bonds.initial_value(select::Radio) = select.default

# ╔═╡ 074386a2-4b6f-4500-9e0c-13f2034c22c6
Bonds.possible_values(select::Radio) = 
	first.(select.options)

# ╔═╡ 47fe62dd-4a60-4dba-b3cf-4a604b96857e
function Bonds.validate_value(select::Radio, val)
	val ∈ (first(p) for p in select.options)
end

# ╔═╡ 15c5572c-26c0-4993-b7a0-3c4ee1de8b87
# ╠═╡ skip_as_script = true
#=╠═╡
Radio(["a", "b", "c"], default="b")
  ╠═╡ =#

# ╔═╡ e21e91a8-3d9c-4505-b78d-2f6f46473905
# ╠═╡ skip_as_script = true
#=╠═╡
Radio(["a", "b", "c"], orientation=:vertical, default="b")
  ╠═╡ =#

# ╔═╡ 6c3835df-fb39-4875-81df-d8180f1dd22a
# ╠═╡ skip_as_script = true
#=╠═╡
Radio(["a", "b", "c"], orientation=:horizontal, default="b")
  ╠═╡ =#

# ╔═╡ 00000000-0000-0000-0000-000000000001
PLUTO_PROJECT_TOML_CONTENTS = """
[deps]
AbstractPlutoDingetjes = "6e696c72-6542-2067-7265-42206c756150"
HypertextLiteral = "ac1192a8-f4b3-4bfe-ba22-af5b92cd3ab2"
Random = "9a3f8284-a2c9-5f02-9a11-845980a1fd5c"

[compat]
AbstractPlutoDingetjes = "~1.1.4"
HypertextLiteral = "~0.9.4"
"""

# ╔═╡ 00000000-0000-0000-0000-000000000002
PLUTO_MANIFEST_TOML_CONTENTS = """
# This file is machine-generated - editing it directly is not advised

julia_version = "1.9.1"
manifest_format = "2.0"
project_hash = "a80718e51710ef42f161d7e7afb0dcfbdded490a"

[[deps.AbstractPlutoDingetjes]]
deps = ["Pkg"]
git-tree-sha1 = "8eaf9f1b4921132a4cff3f36a1d9ba923b14a481"
uuid = "6e696c72-6542-2067-7265-42206c756150"
version = "1.1.4"

[[deps.ArgTools]]
uuid = "0dad84c5-d112-42e6-8d28-ef12dabb789f"
version = "1.1.1"

[[deps.Artifacts]]
uuid = "56f22d72-fd6d-98f1-02f0-08ddc0907c33"

[[deps.Base64]]
uuid = "2a0f44e3-6c83-55bd-87e4-b1978d98bd5f"

[[deps.Dates]]
deps = ["Printf"]
uuid = "ade2ca70-3891-5945-98fb-dc099432e06a"

[[deps.Downloads]]
deps = ["ArgTools", "FileWatching", "LibCURL", "NetworkOptions"]
uuid = "f43a241f-c20a-4ad4-852c-f6b1247861c6"
version = "1.6.0"

[[deps.FileWatching]]
uuid = "7b1f6079-737a-58dc-b8bc-7a2ca5c1b5ee"

[[deps.HypertextLiteral]]
deps = ["Tricks"]
git-tree-sha1 = "c47c5fa4c5308f27ccaac35504858d8914e102f9"
uuid = "ac1192a8-f4b3-4bfe-ba22-af5b92cd3ab2"
version = "0.9.4"

[[deps.InteractiveUtils]]
deps = ["Markdown"]
uuid = "b77e0a4c-d291-57a0-90e8-8db25a27a240"

[[deps.LibCURL]]
deps = ["LibCURL_jll", "MozillaCACerts_jll"]
uuid = "b27032c2-a3e7-50c8-80cd-2d36dbcbfd21"
version = "0.6.3"

[[deps.LibCURL_jll]]
deps = ["Artifacts", "LibSSH2_jll", "Libdl", "MbedTLS_jll", "Zlib_jll", "nghttp2_jll"]
uuid = "deac9b47-8bc7-5906-a0fe-35ac56dc84c0"
version = "7.84.0+0"

[[deps.LibGit2]]
deps = ["Base64", "NetworkOptions", "Printf", "SHA"]
uuid = "76f85450-5226-5b5a-8eaa-529ad045b433"

[[deps.LibSSH2_jll]]
deps = ["Artifacts", "Libdl", "MbedTLS_jll"]
uuid = "29816b5a-b9ab-546f-933c-edad1886dfa8"
version = "1.10.2+0"

[[deps.Libdl]]
uuid = "8f399da3-3557-5675-b5ff-fb832c97cbdb"

[[deps.Logging]]
uuid = "56ddb016-857b-54e1-b83d-db4d58db5568"

[[deps.Markdown]]
deps = ["Base64"]
uuid = "d6f4376e-aef5-505a-96c1-9c027394607a"

[[deps.MbedTLS_jll]]
deps = ["Artifacts", "Libdl"]
uuid = "c8ffd9c3-330d-5841-b78e-0817d7145fa1"
version = "2.28.2+0"

[[deps.MozillaCACerts_jll]]
uuid = "14a3606d-f60d-562e-9121-12d972cd8159"
version = "2022.10.11"

[[deps.NetworkOptions]]
uuid = "ca575930-c2e3-43a9-ace4-1e988b2c1908"
version = "1.2.0"

[[deps.Pkg]]
deps = ["Artifacts", "Dates", "Downloads", "FileWatching", "LibGit2", "Libdl", "Logging", "Markdown", "Printf", "REPL", "Random", "SHA", "Serialization", "TOML", "Tar", "UUIDs", "p7zip_jll"]
uuid = "44cfe95a-1eb2-52ea-b672-e2afdf69b78f"
version = "1.9.0"

[[deps.Printf]]
deps = ["Unicode"]
uuid = "de0858da-6303-5e67-8744-51eddeeeb8d7"

[[deps.REPL]]
deps = ["InteractiveUtils", "Markdown", "Sockets", "Unicode"]
uuid = "3fa0cd96-eef1-5676-8a61-b3b8758bbffb"

[[deps.Random]]
deps = ["SHA", "Serialization"]
uuid = "9a3f8284-a2c9-5f02-9a11-845980a1fd5c"

[[deps.SHA]]
uuid = "ea8e919c-243c-51af-8825-aaa63cd721ce"
version = "0.7.0"

[[deps.Serialization]]
uuid = "9e88b42a-f829-5b0c-bbe9-9e923198166b"

[[deps.Sockets]]
uuid = "6462fe0b-24de-5631-8697-dd941f90decc"

[[deps.TOML]]
deps = ["Dates"]
uuid = "fa267f1f-6049-4f14-aa54-33bafae1ed76"
version = "1.0.3"

[[deps.Tar]]
deps = ["ArgTools", "SHA"]
uuid = "a4e569a6-e804-4fa4-b0f3-eef7a1d5b13e"
version = "1.10.0"

[[deps.Tricks]]
git-tree-sha1 = "aadb748be58b492045b4f56166b5188aa63ce549"
uuid = "410a4b4d-49e4-4fbc-ab6d-cb71b17b3775"
version = "0.1.7"

[[deps.UUIDs]]
deps = ["Random", "SHA"]
uuid = "cf7118a7-6976-5b1a-9a39-7adc72f591a4"

[[deps.Unicode]]
uuid = "4ec0a83e-493e-50e2-b9ac-8f72acf5a8f5"

[[deps.Zlib_jll]]
deps = ["Libdl"]
uuid = "83775a58-1f1d-513f-b197-d71354ab007a"
version = "1.2.13+0"

[[deps.nghttp2_jll]]
deps = ["Artifacts", "Libdl"]
uuid = "8e850ede-7688-5339-a07c-302acd2aaf8d"
version = "1.48.0+0"

[[deps.p7zip_jll]]
deps = ["Artifacts", "Libdl"]
uuid = "3f19e933-33d8-53b3-aaab-bd5110c3b7a0"
version = "17.4.0+0"
"""

# ╔═╡ Cell order:
# ╠═0283cdf9-5dbf-4423-a298-357641f0d480
# ╠═8b2c2972-ca22-41a6-8bd5-bce1431ead8b
# ╠═88f021e1-b626-42a8-b88a-e90cc807fa9f
# ╠═6a14468f-2d51-4630-a4f3-eeae8a7ca912
# ╠═dabfd200-ad92-42f0-ab12-cc75e2253b90
# ╠═4708c993-9cc3-43df-9984-57b97171e1a3
# ╠═081e6a06-0a6d-49b5-ae1a-05c20f0bd924
# ╠═074386a2-4b6f-4500-9e0c-13f2034c22c6
# ╠═47fe62dd-4a60-4dba-b3cf-4a604b96857e
# ╠═15c5572c-26c0-4993-b7a0-3c4ee1de8b87
# ╠═e21e91a8-3d9c-4505-b78d-2f6f46473905
# ╠═6c3835df-fb39-4875-81df-d8180f1dd22a
# ╟─00000000-0000-0000-0000-000000000001
# ╟─00000000-0000-0000-0000-000000000002
